//------------------------------------------------------------------
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"
//------------------------------------------------------------------
#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1  LimeGreen
#property indicator_color2  PaleVioletRed
#property indicator_width1  2
#property indicator_width2  2
#property indicator_minimum 0
#property indicator_maximum 1


//
//
//
//
//

extern string TimeFrame          = "current time frame";
extern int    HmaLength          = 14;
extern int    Price              =  0;
extern double Power              =  2;
extern bool   alertsOn           = false;
extern bool   alertsOnCurrentBar = true;
extern bool   alertsMessage      = true;
extern bool   alertsSound        = false;
extern bool   alertsEmail        = false;
extern bool   Interpolate        = true;

//
//
//
//
//

double hull[];
double hullUp[];
double hullDn[];
double slope[];

int    timeFrame;
string indicatorFileName;
bool   returnBars;
bool   calculateValue;

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(4);
      SetIndexBuffer(0,hullUp); SetIndexStyle(0,DRAW_HISTOGRAM);
      SetIndexBuffer(1,hullDn); SetIndexStyle(1,DRAW_HISTOGRAM);
      SetIndexBuffer(2,hull);
      SetIndexBuffer(3,slope);

      //
      //
      //
      //
      //
         
         indicatorFileName = WindowExpertName();
         calculateValue    = (TimeFrame=="calculateValue"); if (calculateValue) return(0);
         returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
         timeFrame         = stringToTimeFrame(TimeFrame);

      //
      //
      //
      //
      //
   
   IndicatorShortName(timeFrameToString(timeFrame)+" Hull moving average ("+HmaLength+")");
   return(0);
}



//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { hullUp[0] = limit+1; return(0); }

   //
   //
   //
   //
   //

   if (calculateValue || timeFrame==Period())
   {
      int HalfPeriod = MathFloor(HmaLength/2);
      int HullPeriod = MathFloor(MathSqrt(HmaLength));
      
      //
      //
      //
      //
      //
      
      for(int i=limit; i>=0; i--)
      {
         double price     = iMA(NULL,0,1,0,MODE_SMA,Price,i);
                hull[i]   = iLwmp(2.0*iLwmp(price,HalfPeriod,Power,i,0)-iLwmp(price,HmaLength,Power,i,1),HullPeriod,Power,i,2);
                hullUp[i] = EMPTY_VALUE;
                hullDn[i] = EMPTY_VALUE;
                slope[i]  = slope[i+1];
                  if (hull[i]>hull[i+1]) slope[i]=  1;
                  if (hull[i]<hull[i+1]) slope[i]= -1;
                  if (slope[i] ==  1) hullUp[i] = 1;
                  if (slope[i] == -1) hullDn[i] = 1;
      }            
      manageAlerts();
      return(0);
   }
   
   //
   //
   //
   //
   //
   

   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         slope[i]  = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",HmaLength,Price,Power,alertsOn,alertsOnCurrentBar,alertsMessage,alertsSound,alertsEmail,3,y);
         hullUp[i] = EMPTY_VALUE;
         hullDn[i] = EMPTY_VALUE;
            if (slope[i] ==  1) hullUp[i] = 1;
            if (slope[i] == -1) hullDn[i] = 1;
   }
   return(0);         
}

  
//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrentBar)
           int whichBar = 0;
      else     whichBar = 1; 
      if (slope[whichBar] != slope[whichBar+1])
      {
         if (slope[whichBar] ==  1) doAlert(whichBar,"up");
         if (slope[whichBar] == -1) doAlert(whichBar,"down");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  timeFrameToString(Period())+" "+Symbol()+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" Hull parabolic slope changed to "+doWhat;
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Hull parabolic "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}
  
//------------------------------------------------------------------
//                                                                 
//------------------------------------------------------------------
//
//
//
//
//

double workLwmp[][3];
double iLwmp(double price, double period, double power, int r, int instanceNo=0)
{
   if (ArrayRange(workLwmp,0)!= Bars) ArrayResize(workLwmp,Bars); r = Bars-r-1;
   
   //
   //
   //
   //
   //
   
   workLwmp[r][instanceNo] = price;
      double sumw = MathPow(period,power);
      double sum  = sumw*price;
      for(int k=1; k<period && (r-k)>=0; k++)
      {
         double weight = MathPow(period-k,power);
                sumw  += weight;
                sum   += weight*workLwmp[r-k][instanceNo];  
      }             
   return(sum/sumw);
}


//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}